package com.idea.buildManage.excel;

import cn.hutool.core.util.IdUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSONObject;
import com.idea.buildManage.model.MnpBuilding;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.service.MnpBuildingService;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.idea.buildManage.service.ParkRoomService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class ParkRoomListener extends AnalysisEventListener<ParkRoomExcel> {

    private List<ParkRoom> list = new ArrayList<>();

    private List<ParkRoomExcel> excelList = new ArrayList<>();

    private List<ParkRoomExcel> badList = new ArrayList<>();

    Integer count;

    private JSONObject jsonObject;
    private ParkRoomService roomService;
    private MnpBuildingService buildingService;
    private ParkFloorDiscService floorDiscService;
    private ParkInfoService parkInfoService;
    private SysDictService sysDictService;


    public ParkRoomListener(ParkRoomService roomService, MnpBuildingService buildingService, ParkFloorDiscService floorDiscService,
                            ParkInfoService parkInfoService, SysDictService sysDictService, JSONObject jsonObject) {
        this.roomService = roomService;
        this.buildingService = buildingService;
        this.floorDiscService = floorDiscService;
        this.parkInfoService = parkInfoService;
        this.sysDictService = sysDictService;
        this.jsonObject = jsonObject;
    }


    @Override
    public void invoke(ParkRoomExcel parkRoomExcel, AnalysisContext analysisContext) {
        excelList.add(parkRoomExcel);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {


        // 用途字典
        List<Map<String, String>> houseUsage = sysDictService.selectDictList("HOUSE_USAGE");
        // 装修情况字典
        List<Map<String, String>> decorationSituation = sysDictService.selectDictList("DECORATION_SITUATION");
        // 可售状态字典
        List<Map<String, String>> saleStatus = sysDictService.selectDictList("SALE_STATUS");
        // 房屋性质字典
        List<Map<String, String>> groupNature = sysDictService.selectDictList("GROUP_NATURE");


        if (CollectionUtils.isNotEmpty(excelList)) {
            ParkRoom data;
            ParkRoomExcel excel = null;
            Map<String, String> buildNameIdMap = buildingService.getNameIdMap();
            Map<String, String> discNameIdMap = floorDiscService.getNameIdMap();
            Map<String, String> parkNameIdMap = parkInfoService.getNameIdMap();
            for (int i = 0; i < excelList.size(); i++) {

                boolean groupNameFlag = true;
                boolean discNameFlag = true;
                boolean buildNumFlag = true;
                boolean roomNoFlag = true;
                boolean floorFlag = true;
                boolean predictionInternalAreaFlag = true;
                boolean predictionShareAreaFlag = true;
                boolean predictionLandAreaFlag = true;
                boolean predictionBuildAreaFlag = true;
                boolean actualInternalAreaFlag = true;
                boolean actualShareAreaFlag = true;
                boolean actualBuildAreaFlag = true;
                boolean roomUseFlag = true;
                boolean decorationSituationFlag = true;
                boolean houseTypeFlag = true;
                boolean saleStatusFlag = true;
                boolean natureFlag = true;


                StringBuilder builder = new StringBuilder();


                try {
                    data = new ParkRoom();
                    excel = excelList.get(i);
                    if (StringUtils.isEmpty(excel.getGroupName())) {
                        groupNameFlag = false;
                        builder.append("楼盘/小区名称为空").append(",");
                        excel.setGroupName("{" + excel.getGroupName() + "}");
                    }

                    if (StringUtils.isEmpty(excel.getDiscName())) {
                        discNameFlag = false;
                        builder.append("分期名称为空").append(",");
                        excel.setDiscName("{" + excel.getDiscName() + "}");
                    }

                    if (StringUtils.isEmpty(excel.getBuildNum())) {
                        buildNumFlag = false;
                        builder.append("单元楼栋号为空").append(",");
                        excel.setBuildNum("{" + excel.getBuildNum() + "}");
                    }

                    // 判断小区是否存在
                    if (StringUtils.isEmpty(parkNameIdMap.get(excel.getGroupName()))) {
                        groupNameFlag = false;
                        builder.append("楼盘/小区不存在").append(",");
                        excel.setGroupName("{" + excel.getGroupName() + "}");
                    }
                    data.setGroupId(parkNameIdMap.get(excel.getGroupName()));

                    // 判断分期是否存在
                    if (StringUtils.isEmpty(discNameIdMap.get(excel.getDiscName()))) {
                        discNameFlag = false;
                        builder.append("分期不存在").append(",");
                        excel.setDiscName("{" + excel.getDiscName() + "}");
                    }
                    data.setDiscId(discNameIdMap.get(excel.getDiscName()));

                    // 判断楼栋是否存在
                    if (StringUtils.isEmpty(buildNameIdMap.get(excel.getBuildNum()))) {
                        buildNumFlag = false;
                        builder.append("楼栋不存在").append(",");
                        excel.setBuildNum("{" + excel.getBuildNum() + "}");
                    }
                    data.setBuildId(buildNameIdMap.get(excel.getBuildNum()));

                    // 户室号
                    if (StringUtils.isEmpty(excel.getRoomNo())) {
                        roomNoFlag = false;
                        builder.append("户室号为空").append(",");
                        excel.setRoomNo("{" + excel.getRoomNo() + "}");
                    }
                    data.setRoomNo(excel.getRoomNo());

                    // 所在层
                    if (StringUtils.isEmpty(excel.getFloor())) {
                        floorFlag = false;
                        builder.append("所在层为空").append(",");
                        excel.setFloor("{" + excel.getFloor() + "}");
                    }
                    data.setFloor(excel.getFloor());

                    if (StringUtils.isNotEmpty(excel.getPredictionShareArea())) {
                        try {
                            data.setPredictionShareArea(Double.valueOf(excel.getPredictionShareArea()));
                        }catch (NumberFormatException e){
                            predictionShareAreaFlag = false;
                            builder.append("预测分摊面积格式有误").append(",");
                            excel.setPredictionShareArea("{" + excel.getPredictionShareArea() + "}");
                        }
                    }
                    if (StringUtils.isNotEmpty(excel.getPredictionInternalArea())) {
                        try {
                            data.setPredictionInternalArea(Double.valueOf(excel.getPredictionInternalArea()));
                        }catch (NumberFormatException e){
                            predictionInternalAreaFlag = false;
                            builder.append("预测套内面积格式有误").append(",");
                            excel.setPredictionInternalArea("{" + excel.getPredictionInternalArea() + "}");
                        }
                    }
                    if (StringUtils.isNotEmpty(excel.getPredictionLandArea())) {
                        try {
                            data.setPredictionLandArea(Double.valueOf(excel.getPredictionLandArea()));
                        }catch (NumberFormatException e){
                            predictionLandAreaFlag = false;
                            builder.append("预测土地面积格式有误").append(",");
                            excel.setPredictionLandArea("{" + excel.getPredictionLandArea() + "}");
                        }
                    }
                    if (StringUtils.isNotEmpty(excel.getPredictionBuildArea())) {
                        try {
                            data.setPredictionBuildArea(Double.valueOf(excel.getPredictionBuildArea()));
                        }catch (NumberFormatException e){
                            predictionBuildAreaFlag = false;
                            builder.append("预测建筑面积格式有误").append(",");
                            excel.setPredictionBuildArea("{" + excel.getPredictionBuildArea() + "}");
                        }
                    }


                    // 实测套内面积
                    if (StringUtils.isEmpty(excel.getActualInternalArea())) {
                        actualInternalAreaFlag = false;
                        builder.append("实测套内面积为空").append(",");
                        excel.setActualInternalArea("{" + excel.getActualInternalArea() + "}");
                    }
                    try {
                        data.setActualInternalArea(Double.valueOf(excel.getActualInternalArea()));
                    } catch (NumberFormatException e) {
                        actualInternalAreaFlag = false;
                        builder.append("实测套内面积格式有误").append(",");
                        excel.setActualInternalArea("{" + excel.getActualInternalArea() + "}");
                    }

                    // 实测分摊面积
                    if (StringUtils.isEmpty(excel.getActualShareArea())) {
                        actualShareAreaFlag = false;
                        builder.append("实测分摊面积为空").append(",");
                        excel.setActualShareArea("{" + excel.getActualShareArea() + "}");
                    }
                    try {
                        data.setActualShareArea(Double.valueOf(excel.getActualShareArea()));
                    } catch (NumberFormatException e) {
                        actualShareAreaFlag = false;
                        builder.append("实测分摊面积格式有误").append(",");
                        excel.setActualShareArea("{" + excel.getActualShareArea() + "}");
                    }

                    // 实测建筑面积
                    if (StringUtils.isEmpty(excel.getActualBuildArea())) {
                        actualBuildAreaFlag = false;
                        builder.append("实测建筑面积").append(",");
                        excel.setActualBuildArea("{" + excel.getActualBuildArea() + "}");
                    }
                    try {
                        data.setActualBuildArea(Double.valueOf(excel.getActualBuildArea()));
                    } catch (NumberFormatException e) {
                        actualBuildAreaFlag = false;
                        builder.append("预测建筑面积格式有误").append(",");
                        excel.setActualBuildArea("{" + excel.getActualBuildArea() + "}");
                    }


                    // 用途
                    if (StringUtils.isNotEmpty(excel.getRoomUse())) {
                        for (Map<String, String> stringStringMap : houseUsage) {
                            if (stringStringMap.get("label").equals(excel.getRoomUse())) {
                                data.setRoomUse(Integer.valueOf(stringStringMap.get("value")));
                            }
                        }
                        // 没匹配上的情况
                        if (data.getRoomUse() == null) {
                            roomUseFlag = false;
                            builder.append("用途格式有误").append(",");
                            excel.setRoomUse("{" + excel.getRoomUse() + "}");
                        }
                    }

                    // 装修情况
                    if (StringUtils.isNotEmpty(excel.getDecorationSituation())) {
                        for (Map<String, String> stringStringMap : decorationSituation) {
                            if (stringStringMap.get("label").equals(excel.getDecorationSituation())) {
                                data.setDecorationSituation(Integer.valueOf(stringStringMap.get("value")));
                            }
                        }
                        // 没匹配上的情况
                        if (data.getDecorationSituation() == null) {
                            decorationSituationFlag = false;
                            builder.append("装修情况格式有误").append(",");
                            excel.setDecorationSituation("{" + excel.getDecorationSituation() + "}");
                        }
                    }

                    // 装修情况
                    if (StringUtils.isNotEmpty(excel.getNature())) {
                        for (Map<String, String> stringStringMap : groupNature) {
                            if (stringStringMap.get("label").equals(excel.getNature())) {
                                data.setNature(stringStringMap.get("value"));
                            }
                        }
                        // 没匹配上的情况
                        if (data.getNature() == null) {
                            natureFlag = false;
                            builder.append("房屋性质有误").append(",");
                            excel.setNature("{" + excel.getNature() + "}");
                        }
                    }



                    // 可售状态
                    if (StringUtils.isEmpty(excel.getSaleStatus())) {
                        saleStatusFlag = false;
                        builder.append("可售状态为空").append(",");
                        excel.setSaleStatus("{" + excel.getSaleStatus() + "}");
                    }
                    for (Map<String, String> stringStringMap : saleStatus) {
                        if (stringStringMap.get("label").equals(excel.getSaleStatus())) {
                            data.setSaleStatus(Integer.valueOf(stringStringMap.get("value")));
                        }
                    }
                    // 没匹配上的情况
                    if (data.getSaleStatus() == null) {
                        saleStatusFlag = false;
                        builder.append("可售状态格式有误").append(",");
                        excel.setSaleStatus("{" + excel.getSaleStatus() + "}");
                    }

                    if (groupNameFlag && discNameFlag && buildNumFlag && roomNoFlag && floorFlag && actualInternalAreaFlag
                            && actualShareAreaFlag && actualBuildAreaFlag && roomUseFlag && decorationSituationFlag && houseTypeFlag
                            && saleStatusFlag && natureFlag
                    ) {
                        // TODO: 2024/9/11 户型处理
                        data.setId(IdUtil.simpleUUID());
                        data.setCreatedId(ShiroUtils.getUserId());
                        data.setCreatedAt(new Date());
                        data.setRoomNumber(excel.getRoomNumber());

                        data.setTwoPointTwo(excel.getTwoPointTwo());
                        data.setRoomNumber(excel.getRoomNumber());
                        data.setRemark(excel.getRemark());
                        // 默认待售
                        data.setSoldStatus(1);
                        // 选房状态
                        data.setIsChoose(0);
                        data.setDelFlag("0");
                        list.add(data);
                    } else {
                        throw new RuntimeException(builder.toString());
                    }


                } catch (Exception e) {
                    e.printStackTrace();
                    String message = e.getMessage();
                    excel.setReason(message);
                    badList.add(excel);
                }


            }

            count = list.size();
            jsonObject.put("successCount", count);
            jsonObject.put("errorList", badList);
            jsonObject.put("errorCount", badList.size());
            //保存
            saveData();

            excelList.clear();
            list.clear();


        } else {
            throw new RuntimeException("excel无数据");
        }

    }

    private void saveData() {
        if (CollectionUtils.isNotEmpty(list)) {
            roomService.insertBatch(list);
        }
    }


}
