package com.idea.buildManage.excel;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.handler.context.CellWriteHandlerContext;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteTableHolder;
import com.rockstar.util.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFFont;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class WriteHandlerStrategy implements CellWriteHandler {



    @Override
    public void afterCellCreate(CellWriteHandlerContext context) {
        CellWriteHandler.super.afterCellCreate(context);
    }

    @Override
    public void afterCellDispose(CellWriteHandlerContext context) {
        Cell cell = context.getCell();
        // 拿到poi的workbook
        Workbook workbook = context.getWriteWorkbookHolder().getWorkbook();
        CellStyle baseCellStyle = getBaseCellStyle(workbook);
        int rowIndex = cell.getRowIndex();
        // 只处理非表头行
        if (rowIndex == 0) {
            return;
        }
        // 如果内容是字符串类型
        if (cell.getCellType() == CellType.STRING) {
            String stringValue = cell.getStringCellValue();
            // 处理字符串
            // 如果内容中有{},提取到内容 并且计算索引位置 对{}中的值进行颜色设置
            List<String> result = new ArrayList<>();
            Pattern BRACES = Pattern.compile("\\{.*?\\}");
            Matcher matcher = BRACES.matcher(stringValue);
            while (matcher.find()) {
                String group = matcher.group(0);
                // 既然是工具,就尽量不要依赖于外部的包,这里可以用StringUtils.isBlank
                if (StringUtils.isNotEmpty(group)) {
                    result.add(group);
                }
            }
            // result 不为空 即为有需要改色的内容
            Font font = workbook.createFont();
            if (CollUtil.isNotEmpty(result)) {
                stringValue = stringValue.replaceAll("[{}]", "");
                if("null".equals(stringValue)){
                    // 设置背景色
                    baseCellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
                    baseCellStyle.setFillForegroundColor(IndexedColors.RED.getIndex());
                }
                // 再设置回每个单元格里
                cell.setCellValue(stringValue);
                // 设置字体
                font.setColor(IndexedColors.RED.getIndex());
                baseCellStyle.setFont(font);
                cell.setCellStyle(baseCellStyle);
            }
        }

        // 这里要把 WriteCellData的样式清空， 不然后面还有一个拦截器 FillStyleCellWriteHandler 默认会将 WriteCellStyle 设置到
        // cell里面去 会导致自己设置的不一样（很关键）
        context.getFirstCellData().setWriteCellStyle(null);


    }

    private CellStyle getBaseCellStyle(Workbook workbook) {
        CellStyle style = workbook.createCellStyle();
        // 设置表格内容垂直居中
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        // 设置表格内容水平居中
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setWrapText(true);
        return style;
    }

    private void setRowFontColor(Row row, short colorIndex) {
        Workbook workbook = row.getSheet().getWorkbook();
        Font font = workbook.createFont();
        font.setColor(colorIndex);

        for (int i = 0; i <= row.getLastCellNum(); i++) {
            Cell cell = row.getCell(i);
            if (cell == null) {
                cell = row.createCell(i);
            }
            CellStyle style = cell.getCellStyle();
            if (style == null) {
                style = workbook.createCellStyle();
            }
            style.setFont(font);
            cell.setCellStyle(style);
        }
    }


    // 此方法必须重写 不然单元格样式有可能不生效
    @Override
    public int order() {
        return 50001;
    }


}
