package com.idea.buyingMore.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buyingMore.dto.BuyingMoreDto;
import com.idea.buyingMore.mapper.BuyingMoreExtendMapper;
import com.idea.buyingMore.mapper.BuyingMoreMapper;
import com.idea.buyingMore.model.BuyingMore;
import com.idea.buyingMore.model.BuyingMoreExample;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.mapper.ContractManageMapper;
import com.idea.customerManagement.model.ContractManage;
import com.idea.util.DateUtils;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.FrameUserExample;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Service
public class BuyingMoreService implements BaseService<BuyingMore, BuyingMoreExample> {

    @Autowired
    private BuyingMoreMapper modelMapper;
    @Autowired
    private ContractManageMapper contractManageMapper;
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private BuyingMoreExtendMapper extendMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(BuyingMore buyingMore) {
        buyingMore.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(buyingMore);
    }

    public int insertWithoutId(BuyingMore buyingMore) {
        return modelMapper.insertSelective(buyingMore);
    }

    @Override
    public BuyingMore selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(BuyingMore buyingMore) {
        return modelMapper.updateByPrimaryKeySelective(buyingMore);
    }

    @Override
    public int updateByExampleSelective(BuyingMore buyingMore, BuyingMoreExample buyingMoreExample) {
        return modelMapper.updateByExampleSelective(buyingMore, buyingMoreExample);
    }

    @Override
    public int updateByExample(BuyingMore buyingMore, BuyingMoreExample buyingMoreExample) {
        return modelMapper.updateByExample(buyingMore, buyingMoreExample);
    }

    @Override
    public List<BuyingMore> selectByExample(BuyingMoreExample buyingMoreExample) {
        return modelMapper.selectByExample(buyingMoreExample);
    }

    @Override
    public long countByExample(BuyingMoreExample buyingMoreExample) {
        return modelMapper.countByExample(buyingMoreExample);
    }

    @Override
    public int deleteByExample(BuyingMoreExample buyingMoreExample) {
        return modelMapper.deleteByExample(buyingMoreExample);
    }

    /**
     * 新增
     *
     * @param model
     * @return
     */
    public int add(BuyingMore model) {

        String serialNumber = getSerialNumber(model.getType());
        model.setDelFlag(0);
        model.setSerialNumber(serialNumber);
        model.setCreatedId(ShiroUtils.getUserId());
        model.setCreatedAt(new Date());
        int result = insertSelective(model);
        return result;
    }

    /**
     * 编辑
     *
     * @param model
     * @return
     */
    public int edit(BuyingMore model) {
        model.setUpdatedId(ShiroUtils.getUserId());
        model.setUpdatedAt(new Date());
        int result = modelMapper.updateByPrimaryKeySelective(model);
        return result;
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    public int delete(String id) {
        int result = 0;
        BuyingMore buyingMore = modelMapper.selectByPrimaryKey(id);
        if (ObjectUtil.isNotNull(buyingMore)) {
            buyingMore.setDelFlag(1);
            result = modelMapper.updateByPrimaryKeySelective(buyingMore);
        }
        return result;
    }

    public String getSerialNumber(Integer type) {

        BuyingMoreExample example = new BuyingMoreExample();
        example.createCriteria().andTypeEqualTo(type);
        Long count = countByExample(example);
        String number = "";
        if (count.doubleValue() == 0) {
            number = "0001";
        } else {
            count++;
            number = String.format("%04d", count);
        }
        String year = DateUtils.getYear(new Date(), 0);
        String serialNumber = "";
        if (type == 1) {
            serialNumber = "ZG-" + year + number;
        }
        if (type == 2) {
            serialNumber = "HG-" + year + number;
        }
        return serialNumber;
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<BuyingMoreDto> list(Tablepar tablepar, BuyingMoreDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<BuyingMoreDto> list = extendMapper.listByModel(model);
        for(BuyingMoreDto buyingMore : list){
            // 转让前 买受人占比
            BigDecimal buyerProportion = new BigDecimal(buyingMore.getBuyerProportion().toString());
            // 转让前 安居公司占比 100-买房人占比
            BigDecimal companyProportion = new BigDecimal("100").subtract(buyerProportion);
            // 转让后 买受人占比
            Double buyerProportionAfter = buyingMore.getTransferBuyerProportionAfter();
            // 转让后 安居公司占比
            Double companyProportionAfter = buyingMore.getTransferCompanyProportionAfter();

            String transferProportionBefore = "买受人" + buyerProportion + "%;" + "安居公司" + companyProportion + "%";
            String transferProportionAfter = "买受人" + buyerProportionAfter + "%;" + "安居公司" + companyProportionAfter + "%";
            buyingMore.setTransferProportionBeforeStr(transferProportionBefore);
            buyingMore.setTransferProportionAfterStr(transferProportionAfter);
        }
        PageInfo<BuyingMoreDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<BuyingMore> listAll(BuyingMore model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private BuyingMoreExample getCondition(BuyingMore model, DateTrans dt) {
        BuyingMoreExample ex = new BuyingMoreExample();
        BuyingMoreExample.Criteria criteria = ex.createCriteria();
        criteria.andDelFlagEqualTo(0);
        if (StringUtils.isNotEmpty(model.getBuyerName())) {
            criteria.andBuyerNameLike("%" + model.getBuyerName() + "%");
        }
        if (StringUtils.isNotEmpty(model.getHouseName())) {
            criteria.andHouseNameLike("%" + model.getHouseName() + "%");
        }
        return ex;
    }

    /**
     * 根据合同id 查询
     * @param contractId
     * @return
     */
    public BuyingMoreDto getByContractId(String contractId) {

        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(contractId);
        BuyingMoreDto result = new BuyingMoreDto();
        result.setContractId(contractId);
        result.setHouseId(contractManage.getHouseId());
        result.setBuyerName(contractManage.getBuyerName());
        result.setHouseName(contractManage.getHouseName());
        result.setActualBuildArea(contractManage.getActualBuildArea());
        result.setSigningDate(contractManage.getSigningDate());
        result.setHousePrice(contractManage.getHousePrice());
        // 买受人最新份额占比 = 合同中买受人产权份额占比+已增购的历史记录份额-已回购的历史份额
        String allTransferProportion = extendMapper.getAllTransferProportion(contractManage.getHouseId());
        BigDecimal baseBuyerProportion = new BigDecimal(contractManage.getBuyerProportion().toString());
        BigDecimal buyerProportion = baseBuyerProportion.add(new BigDecimal(allTransferProportion));
        result.setBuyerProportion(buyerProportion.doubleValue());
        return result;

    }

    public BuyingMoreDto getById(String id) {

        BuyingMore buyingMore = selectByPrimaryKey(id);
        String truename = "";
        FrameUser frameUser = frameUserService.selectByPrimaryKey(buyingMore.getCreatedId());
        if(ObjectUtil.isNotNull(frameUser)){
            truename = frameUser.getTruename();
        }
        BuyingMoreDto result = new BuyingMoreDto();
        BeanUtil.copyProperties(buyingMore, result);
        result.setCreatedName(truename);
        return result;
    }


}
