package com.idea.customerManagement.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.excel.MnpBuildingExcel;
import com.idea.buildManage.excel.MnpBuildingListener;
import com.idea.buildManage.excel.ParkRoomExcel;
import com.idea.customerManagement.dto.CustomerManagementDto;
import com.idea.customerManagement.dto.RoomSelectionInfoDto;
import com.idea.customerManagement.excel.CustomerManagementExcel;
import com.idea.customerManagement.service.CustomerManagementService;
import com.idea.util.ExcelUtils;
import com.rockstar.common.base.BaseController;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.List;

@Controller
@RequestMapping(value = "CustomerManagementController")
@Api(tags = "客户管理")
public class CustomerManagementController extends BaseController {

    @Autowired
    private CustomerManagementService modelService;

    /**
     * 分页
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    @PostMapping(value = "list",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object list(Tablepar tablepar, CustomerManagementDto model, DateTrans dt){
        PageInfo<CustomerManagementDto> page= modelService.listByModel(tablepar,model,dt);
        TableSplitResult<CustomerManagementDto> result = new TableSplitResult<CustomerManagementDto>(page.getPageNum(), page.getTotal(), page.getList());
        return  result;
    }

    /**
     * 新增轮号客户
     * @param record
     * @return
     */
    @PostMapping(value = "add",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult add(CustomerManagementDto record){
        int result = modelService.add(record);
        return result(result);
    }

    /**
     * 编辑轮号客户
     * @param record
     * @return
     */
    @PostMapping(value = "edit",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult edit(CustomerManagementDto record){
        int result = modelService.edit(record);
        return result(result);
    }

    /**
     * 删除轮号客户
     * @param ids
     * @return
     */
    @PostMapping(value = "delete",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult delete(String ids){
        int result = modelService.delete(ids);
        return result(result);
    }

    @PostMapping(value = "getById",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object getById(String id){
        return modelService.getById(id);
    }


    /**
     * 放弃资格
     * @param record
     * @return
     */
    @PostMapping(value = "giveUp",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult giveUp(CustomerManagementDto record){
        int result = modelService.giveUp(record);
        return result(result);
    }

    /**
     * 选房通知单 下载
     * @param id
     * @param response
     */
    @GetMapping(value = "download",produces = {"application/json;charset=UTF-8"})
    public void download(String id, HttpServletResponse response) {
        modelService.downLoad(id,response);
    }

    @PostMapping(value = "handleImport", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult handleImport(@RequestParam("file") MultipartFile file) throws IOException {
        JSONObject jsonObject = modelService.handleImport(file);
        return AjaxResult.success(jsonObject);
    }


    /**
     * 导出
     * @param list
     * @param response
     * @throws IOException
     */
    @PostMapping(value = "errorListExport")
    @ResponseBody
    public void errorListExport(@RequestBody List<CustomerManagementExcel> list, HttpServletResponse response) throws IOException {
        modelService.errorListExport(list,response);
    }


    @PostMapping(value = "downLoadTemplate")
    @ResponseBody
    public void downLoadTemplate(HttpServletResponse response) throws IOException {
        modelService.downLoadTemplate(response);
    }

}
