package com.idea.customerManagement.service;

import cn.hutool.core.util.IdUtil;
import com.idea.customerManagement.mapper.BuyerExtendMapper;
import com.idea.customerManagement.mapper.BuyerMapper;
import com.idea.customerManagement.model.Buyer;
import com.idea.customerManagement.model.BuyerExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class BuyerService implements BaseService<Buyer, BuyerExample> {

    @Autowired
    private BuyerMapper modelMapper;
    @Autowired
    private BuyerExtendMapper extendMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(Buyer buyer) {
        buyer.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(buyer);
    }

    @Override
    public Buyer selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(Buyer buyer) {
        return modelMapper.updateByPrimaryKeySelective(buyer);
    }

    @Override
    public int updateByExampleSelective(Buyer buyer, BuyerExample buyerExample) {
        return modelMapper.updateByExampleSelective(buyer,buyerExample);
    }

    @Override
    public int updateByExample(Buyer buyer, BuyerExample buyerExample) {
        return modelMapper.updateByExample(buyer,buyerExample);
    }

    @Override
    public List<Buyer> selectByExample(BuyerExample buyerExample) {
        return modelMapper.selectByExample(buyerExample);
    }

    @Override
    public long countByExample(BuyerExample buyerExample) {
        return modelMapper.countByExample(buyerExample);
    }

    @Override
    public int deleteByExample(BuyerExample buyerExample) {
        return modelMapper.deleteByExample(buyerExample);
    }

    public List<Buyer> listAll(Buyer model,DateTrans dt){
        List<Buyer> buyers = modelMapper.selectByExample(getCondition(model, dt));
        return buyers;
    }

    public BuyerExample getCondition(Buyer model, DateTrans dt){

        BuyerExample example = new BuyerExample();
        example.setOrderByClause("order_num asc");
        BuyerExample.Criteria criteria = example.createCriteria();
        if(StringUtils.isNotEmpty(model.getCustomerManagementId())){
            criteria.andCustomerManagementIdEqualTo(model.getCustomerManagementId());
        }
        return example;
    }

    /**
     * 查询所有身份证号 去重
     * @return
     */
    public List<String> numberList() {
        List<String> numberList = extendMapper.idNumberList();
        return numberList;
    }

    /**
     * 获取客户id和客户名称
     * @return
     */
    public Map<String,String> getCusIdMap(){
        Map<String, String> map = new HashMap<>();
        List<Buyer> buyers = extendMapper.getCusIdMap();
        if(CollectionUtils.isNotEmpty(buyers)){
            map = buyers.stream().collect(Collectors.toMap(Buyer::getCustomerManagementId,Buyer::getName));
        }
        return map;
    }



}
