package com.idea.customerManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkRoomService;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.dto.CustomerManagementDto;
import com.idea.customerManagement.mapper.ContractManageExtendMapper;
import com.idea.customerManagement.mapper.ContractManageMapper;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.invoice.mapper.InvoiceBankMapper;
import com.idea.invoice.model.InvoiceBank;
import com.idea.invoice.model.InvoiceBankExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Service
public class ContractManageService implements BaseService<ContractManage, ContractManageExample> {

    @Autowired
    private ContractManageMapper modelMapper;
    @Autowired
    private ContractManageExtendMapper extendMapper;
    @Autowired
    private ParkRoomService parkRoomService;
    @Autowired
    private ParkFloorDiscService parkFloorDiscService;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private SysDictService sysDictService;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private InvoiceBankMapper invoiceBankMapper;

    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(ContractManage contractManage) {
        contractManage.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(contractManage);
    }

    public int insertWithoutId(ContractManage contractManage) {
        return modelMapper.insertSelective(contractManage);
    }

    @Override
    public ContractManage selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(ContractManage contractManage) {
        return modelMapper.updateByPrimaryKeySelective(contractManage);
    }

    @Override
    public int updateByExampleSelective(ContractManage contractManage, ContractManageExample contractManageExample) {
        return modelMapper.updateByExampleSelective(contractManage,contractManageExample);
    }

    @Override
    public int updateByExample(ContractManage contractManage, ContractManageExample contractManageExample) {
        return modelMapper.updateByExample(contractManage,contractManageExample);
    }

    @Override
    public List<ContractManage> selectByExample(ContractManageExample contractManageExample) {
        return modelMapper.selectByExample(contractManageExample);
    }

    @Override
    public long countByExample(ContractManageExample contractManageExample) {
        return modelMapper.countByExample(contractManageExample);
    }

    @Override
    public int deleteByExample(ContractManageExample contractManageExample) {
        return modelMapper.deleteByExample(contractManageExample);
    }


    public PageInfo<ContractManageDto> listByModel(Tablepar tablepar, ContractManageDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ContractManageDto> list = extendMapper.listByModel(model);
        PageInfo<ContractManageDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    public List<ContractManageDto> listAll(ContractManageDto model){
        List<ContractManageDto> list = extendMapper.listByModel(model);
        return list;
    }





    /**
     * 认购收取后 新增合同
     * @param roomSelectionInfo
     * @return
     */
    public int addContract(RoomSelectionInfo roomSelectionInfo,String contractId,String userId){

        ParkRoom parkRoom = parkRoomService.selectByPrimaryKey(roomSelectionInfo.getHouseId());
        ParkFloorDisc parkFloorDisc = parkFloorDiscService.selectByPrimaryKey(parkRoom.getDiscId());

        if(StringUtils.isEmpty(contractId)){
            contractId =  IdUtil.simpleUUID();
        }
        if(StringUtils.isEmpty(userId)){
            userId = ShiroUtils.getUserId();
        }
        ContractManage contractManage = new ContractManage();
        contractManage.setId(contractId);
        contractManage.setCustomerManagementId(roomSelectionInfo.getCustomerManagementId());
        contractManage.setHouseId(roomSelectionInfo.getHouseId());
        contractManage.setSubscriptionFunds(roomSelectionInfo.getReceivableMoney());
        contractManage.setActualBuildArea(parkRoom.getActualBuildArea());
        contractManage.setActualInternalArea(parkRoom.getActualInternalArea());
        contractManage.setActualShareArea(parkRoom.getActualShareArea());
        contractManage.setMaintenanceFunds(parkFloorDisc.getMaintenanceFunds());
        if(parkFloorDisc.getMaintenanceFunds() != null){
            BigDecimal area = new BigDecimal(parkRoom.getActualBuildArea().toString());
            contractManage.setMaintenanceTotalPrice(parkFloorDisc.getMaintenanceFunds().multiply(area));
        }
        contractManage.setContractStatus(1);
        contractManage.setFundCollectionStatus(1);
        contractManage.setCreatedAt(new Date());
        contractManage.setCreatedId(userId);
        
        // 拼接买受人名称
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(roomSelectionInfo.getCustomerManagementId());
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        StringBuilder builder = new StringBuilder();
        for(Buyer buyer : buyers){
            if(buyer.getOrderNum() == 1){
                builder.append(buyer.getName()).append(",");
            }else {
                String dictLabel = sysDictService.getDictLabel("RELATIONSHIP", buyer.getRelationship());
                builder.append(buyer.getName())
                        .append("(")
                        .append(dictLabel)
                        .append(")")
                        .append(",");
            }
        }
        if(builder.length() > 0){
            builder.deleteCharAt(builder.length()-1);
        }
        contractManage.setBuyerName(builder.toString());

        ParkRoomResponse response = parkRoomService.getFullName(roomSelectionInfo.getHouseId());
        String fullName = response.getGroupName() + "-" +
                response.getDiscName() + "-" +
                response.getBuildName() + "-" +
                response.getRoomNo();
        contractManage.setHouseName(fullName);
        int result = insertWithoutId(contractManage);
        return result;
    }

    /**
     * 编辑合同
     * @param model
     * @return
     */
    public int edit(ContractManage model){

        model.setUpdatedAt(new Date());
        model.setUpdatedId(ShiroUtils.getUserId());
        // 如果是已签约
        if(model.getContractStatus() == 2){
            // 定金转房款 部分收款
            model.setCollectionStatus(1);
            // 更新房间已售状态
            String houseId = model.getHouseId();
            ParkRoom parkRoom = parkRoomService.selectByPrimaryKey(houseId);
            parkRoom.setSoldStatus(3);
            parkRoomService.updateByPrimaryKeySelective(parkRoom);
            // 更新客户管理 购房状态 已签约
            CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
            customerManagement.setStatus(3);
            customerManagementMapper.updateByPrimaryKeySelective(customerManagement);
            // 更新定金管理 收取状态
            RoomSelectionInfoExample example = new RoomSelectionInfoExample();
            example.createCriteria().andCustomerManagementIdEqualTo(model.getCustomerManagementId());
            List<RoomSelectionInfo> list = roomSelectionInfoMapper.selectByExample(example);
            for(RoomSelectionInfo roomSelectionInfo : list){
                // 已收取或意向金转入 状态的
                if(roomSelectionInfo.getStatus() == 2 || roomSelectionInfo.getStatus() == 3){
                    // 已签约转房款
                    roomSelectionInfo.setStatus(4);
                    roomSelectionInfoMapper.updateByPrimaryKeySelective(roomSelectionInfo);
                }
            }
        }
        int result = updateByPrimaryKeySelective(model);
        return result;
    }


    public ContractManageDto getById(String id) {
        ContractManage contractManage = selectByPrimaryKey(id);
        ContractManageDto result = new ContractManageDto();
        BeanUtil.copyProperties(contractManage,result);
        return result;
    }

    /**
     * 获取合同对应的银行账号
     * @param contractId
     * @return
     */
    public List<InvoiceBank> getBankList(String contractId){

//        ContractManage contractManage = selectByPrimaryKey(contractId);
//        ParkRoom parkRoom = parkRoomService.selectByPrimaryKey(contractManage.getHouseId());
        InvoiceBankExample example = new InvoiceBankExample();
//        example.createCriteria().andSuitCostTypeLike("%" + parkRoom.getDiscId() + "%");
        return invoiceBankMapper.selectByExample(example);
        
    }


}
