package com.idea.my.service;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.my.mapper.MyMenuMapper;
import com.idea.my.model.MyMenu;
import com.idea.my.model.MyMenuExample;
import com.rockstar.biz.model.BizUser;
import com.rockstar.biz.model.BizUserExample;
import com.rockstar.biz.service.BizUserService;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.FramePermission;
import com.rockstar.frame.model.FramePermissionExample;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FramePermissionService;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class MyMenuService implements BaseService<MyMenu, MyMenuExample> {

    //文件mapper
    @Autowired
    private MyMenuMapper modelMapper;

    @Autowired
    private FramePermissionService framePermissionService;

    @Autowired
    private BizUserService bizUserService;

    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        MyMenuExample example = new MyMenuExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public MyMenu selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }
    @Override
    public int updateByPrimaryKeySelective(MyMenu record) {
        if(StringUtils.isEmpty(record.getId())){
            record.setId(SnowflakeIdWorker.getUUID());
            record.setCreatedAt(new Date());
            return modelMapper.insert(record);
        }
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(MyMenu record, MyMenuExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(MyMenu record, MyMenuExample example) {
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<MyMenu> selectByExample(MyMenuExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(MyMenuExample example) {
        return modelMapper.countByExample(example);
    }

    @Override
    public int deleteByExample(MyMenuExample example) {
        return modelMapper.deleteByExample(example);
    }


    public int insertWithoutId(MyMenu record){
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(MyMenu record) {
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insertSelective(record);
    }

    public int insert(MyMenu record){
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insert(record);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<MyMenu> list(Tablepar tablepar, MyMenu model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<MyMenu> list = modelMapper.selectByExample(getCondition(model,dt));
        PageInfo<MyMenu> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }
    /**
     * 查询指定
     *
     * @return
     */
    public Map<String,Object> listAll(MyMenu model, DateTrans dt) {
        Map<String,Object> map = new HashMap<>();
        List<FramePermission> list = new ArrayList<>();
        List<MyMenu> myMenus = modelMapper.selectByExample(getCondition(model, dt));
        if(myMenus!=null&&myMenus.size()>0){
            MyMenu myMenu = myMenus.get(0);
            if(StringUtils.isNotEmpty(myMenu.getMenuId())&&myMenu.getMenuId().length()>2){
                List lists = new ArrayList();
                JSONArray jsonArray = JSONUtil.parseArray(myMenu.getMenuId());
//                JSONArray jsonArray = JSONArray.fromObject(myMenu.getMenuId());//把String转换为json
//                lists = JSONArray.toList(jsonArray);
                lists = JSONUtil.toList(jsonArray, String.class);
                FramePermissionExample example = new FramePermissionExample();
                example.createCriteria().andIdIn(lists);
                list = framePermissionService.selectByExample(example);
            }
        }
        map.put("myPermission",list);
        List<FramePermission>  listAll = framePermissionService.selectByExample(new FramePermissionExample());
        map.put("allPermission",listAll);
        return map;
    }

    public MyMenu getMyMenu(MyMenu model){
        MyMenu myMenu = new MyMenu();
        List<MyMenu> myMenus = modelMapper.selectByExample(getCondition(model, null));
        if(myMenus!=null&&myMenus.size()>0){
            myMenu = myMenus.get(0);
        }
        return myMenu;
    }

    private MyMenuExample getCondition(MyMenu model, DateTrans dt) {
        MyMenuExample ex = new MyMenuExample();
        if(null == model){
            return ex;
        }
        ex.setOrderByClause("created_at desc");
        MyMenuExample.Criteria criteria = ex.createCriteria();
        if(StringUtils.isNotEmpty(model.getUserId())){
            criteria.andUserIdEqualTo(model.getUserId());
        }
        return ex;
    }

    public List<BizUser> bizUserListAll(String mainId){
        BizUserExample example = new BizUserExample();
        example.createCriteria().andMainIdEqualTo(mainId);
        return bizUserService.selectByExample(example);
    }

}
