package com.idea.transactionRecordManage.service;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkInfoExample;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.IntentionalDepositDto;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.IntentionalDepositMapper;
import com.idea.customerManagement.model.*;
import com.idea.customerManagement.service.BuyerService;
import com.idea.invoice.mapper.InvoiceManageMapper;
import com.idea.invoice.model.InvoiceManage;
import com.idea.invoice.model.InvoiceManageExample;
import com.idea.invoice.util.InvoiceConstant;
import com.idea.invoice.util.InvoiceUtil;
import com.idea.paymentManagement.dto.RefundManageDto;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.mapper.RefundDetailMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.paymentManagement.model.PayLogExample;
import com.idea.paymentManagement.model.RefundDetail;
import com.idea.paymentManagement.model.RefundDetailExample;
import com.idea.paymentManagement.service.PayLogService;
import com.idea.transactionRecordManage.mapper.ConvertRecordMapper;
import com.idea.transactionRecordManage.mapper.TransactionRecordMapper;
import com.idea.transactionRecordManage.model.ConvertRecord;
import com.idea.transactionRecordManage.vo.NccRecordVo;
import com.idea.util.DateUtils;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.MD5Util;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TransactionRecordService {

    @Autowired
    private TransactionRecordMapper modelMapper;
    @Autowired
    private SysDictService sysDictService;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private PayLogMapper payLogMapper;
    @Autowired
    private RefundDetailMapper refundDetailMapper;
    @Autowired
    private IntentionalDepositMapper intentionalDepositMapper;
    @Autowired
    private InvoiceManageMapper invoiceManageMapper;
    @Autowired
    private ParkInfoMapper parkInfoMapper;
    @Autowired
    private ParkRoomExtendMapper parkRoomExtendMapper;
    @Autowired
    private ConvertRecordMapper convertRecordMapper;

    /**
     * 收款记录
     *
     * @param model
     * @return
     */
    public PageInfo<NccRecordVo> collectionRecord(Tablepar tablepar, NccRecordVo model) {

        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        if (StringUtils.isNotEmpty(model.getDiscIds())) {
            List<String> list = Arrays.asList(model.getDiscIds().split(","));
            model.setDiscIdList(list);
        }
        // 收款记录
        List<NccRecordVo> list = modelMapper.collectionRecord(model);
        for (NccRecordVo recordVo : list) {
            // 已推送
            if (recordVo.getNcSubmitStatus() != null && recordVo.getNcSubmitStatus() == 1) {
                JSONObject jsonObject = JSONObject.parseObject(recordVo.getNcJson());
                String code = jsonObject.getJSONObject("data").getJSONObject("ar_gatherbill").getString("code");
                recordVo.setNcCode(code);
            }
            // 推送失败
            if (recordVo.getNcSubmitStatus() != null && recordVo.getNcSubmitStatus() == 2) {
                JSONObject jsonObject = JSONObject.parseObject(recordVo.getNcJson());
                String message = jsonObject.getString("message");
                recordVo.setErrorMessage(message);
            }
        }
        PageInfo<NccRecordVo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    public List<NccRecordVo> collectionRecordListAll(NccRecordVo model) {

        List<Map<String, String>> contentType = sysDictService.selectDictList("CONTENT_TYPE");
        // 收款记录
        List<NccRecordVo> list = modelMapper.collectionRecord(model);
//        for (NccRecordVo recordVo : list) {
//            for(Map<String, String> map : contentType){
//                if(recordVo.getContentType().equals(map.get("value"))){
//                    recordVo.setContentType(map.get("label"));
//                }
//            }
//        }
        return list;
    }

    /**
     * 退款记录
     *
     * @param tablepar
     * @param model
     * @return
     */
    public PageInfo<NccRecordVo> refundRecord(Tablepar tablepar, NccRecordVo model) {

//        List<Map<String, String>> contentType = sysDictService.selectDictList("CONTENT_TYPE");
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        if (StringUtils.isNotEmpty(model.getDiscIds())) {
            List<String> list = Arrays.asList(model.getDiscIds().split(","));
            model.setDiscIdList(list);
        }
        // 退款记录
        List<NccRecordVo> list = modelMapper.reFundRecord(model);
        for (NccRecordVo recordVo : list) {
            // 已推送
            if (recordVo.getNcSubmitStatus() != null && recordVo.getNcSubmitStatus() == 1) {
                JSONObject jsonObject = JSONObject.parseObject(recordVo.getNcJson());
                String code = jsonObject.getJSONObject("data").getJSONObject("ap_paybill").getString("code");
                recordVo.setNcCode(code);
            }
            // 推送失败
            if (recordVo.getNcSubmitStatus() != null && recordVo.getNcSubmitStatus() == 2) {
                JSONObject jsonObject = JSONObject.parseObject(recordVo.getNcJson());
                String message = jsonObject.getString("message");
                recordVo.setErrorMessage(message);
            }
        }
        PageInfo<NccRecordVo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    public List<NccRecordVo> refundRecordListAll(NccRecordVo model) {

//        List<Map<String, String>> contentType = sysDictService.selectDictList("CONTENT_TYPE");
        if (StringUtils.isNotEmpty(model.getDiscIds())) {
            List<String> list = Arrays.asList(model.getDiscIds().split(","));
            model.setDiscIdList(list);
        }
        // 收款记录
        List<NccRecordVo> list = modelMapper.reFundRecord(model);
//        for (NccRecordVo recordVo : list) {
//            for(Map<String, String> map : contentType){
//                if(recordVo.getContentType().equals(map.get("value"))){
//                    recordVo.setContentType(map.get("label"));
//                }
//            }
//        }
        return list;
    }

    /**
     * 转换记录
     *
     * @param tablepar
     * @param model
     * @return
     */
    public PageInfo<NccRecordVo> convertRecord(Tablepar tablepar, NccRecordVo model) {

        // 查询房间名称
        List<ParkRoomResponse> fullNames = parkRoomExtendMapper.getFullNames(null);
        Map<String, String> nameMap = new HashMap<>();
        for (ParkRoomResponse response : fullNames) {
            nameMap.put(response.getId(), response.getBuildName() + "-" + response.getRoomNo());
        }

        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        if (StringUtils.isNotEmpty(model.getDiscIds())) {
            List<String> list = Arrays.asList(model.getDiscIds().split(","));
            model.setDiscIdList(list);
        }
        // 转换记录
        List<NccRecordVo> list = modelMapper.convertRecord(model);
        for (NccRecordVo recordVo : list) {
            if (nameMap.containsKey(recordVo.getHouseId())) {
                recordVo.setHouseName(nameMap.get(recordVo.getHouseId()));
            }
            // 已推送
            if (recordVo.getNcSubmitStatus() != null && recordVo.getNcSubmitStatus() == 1) {
                JSONObject jsonObject = JSONObject.parseObject(recordVo.getNcJson());
                String code = jsonObject.getJSONObject("data").getJSONObject("ar_gatherbill").getString("code");
                recordVo.setNcCode(code);
            }
            // 推送失败
            if (recordVo.getNcSubmitStatus() != null && recordVo.getNcSubmitStatus() == 2) {
                JSONObject jsonObject = JSONObject.parseObject(recordVo.getNcJson());
                String message = jsonObject.getString("message");
                recordVo.setErrorMessage(message);
            }
        }

        PageInfo<NccRecordVo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    public List<NccRecordVo> convertRecordListAll(NccRecordVo model) {

        // 查询房间名称
        List<ParkRoomResponse> fullNames = parkRoomExtendMapper.getFullNames(null);
        Map<String, String> nameMap = new HashMap<>();
        for (ParkRoomResponse response : fullNames) {
            nameMap.put(response.getId(), response.getBuildName() + "-" + response.getRoomNo());
        }

        if (StringUtils.isNotEmpty(model.getDiscIds())) {
            List<String> list = Arrays.asList(model.getDiscIds().split(","));
            model.setDiscIdList(list);
        }
        // 收款记录
        List<NccRecordVo> list = modelMapper.convertRecord(model);
        for (NccRecordVo vo : list) {
            if (nameMap.containsKey(vo.getHouseId())) {
                vo.setHouseName(nameMap.get(vo.getHouseId()));
            }
        }
        return list;
    }


    /**
     * 收款记录 ncc提交
     *
     * @param collectionDateFrom
     * @param collectionDateTo
     * @return
     */
    @Synchronized
    public int collectionRecordSubmit(String ids, String collectionDateFrom, String collectionDateTo) {

//        List<PayLog> payLogs = modelMapper.listByDate(collectionDateFrom, collectionDateTo);
        PayLogExample payLogExample = new PayLogExample();
        payLogExample.createCriteria().andIdIn(Arrays.asList(ids.split(",")));
        List<PayLog> payLogs = payLogMapper.selectByExample(payLogExample);
        List<CustomerManagement> customerManagements = customerManagementMapper.selectByExample(new CustomerManagementExample());
        Map<String, CustomerManagement> customerMap = customerManagements.stream()
                .collect(Collectors.toMap(CustomerManagement::getId, item -> item));
        List<ParkInfo> parkInfos = parkInfoMapper.selectByExample(new ParkInfoExample());
        Map<String, ParkInfo> parkInfoMap = parkInfos.stream().collect(Collectors.toMap(ParkInfo::getId, item -> item));
        for (PayLog payLog : payLogs) {
            try {

                if (customerMap.containsKey(payLog.getCustomerManagementId())) {
                    CustomerManagement customerManagement = customerMap.get(payLog.getCustomerManagementId());
                    // 项目档案编码
                    String def10 = "";
                    if (parkInfoMap.containsKey(customerManagement.getGroupId())) {
                        def10 = parkInfoMap.get(customerManagement.getGroupId()).getArchiveCode();
                    }
                    // 组织机构编码
                    String pk_org = "";
                    if (parkInfoMap.containsKey(customerManagement.getGroupId())) {
                        pk_org = parkInfoMap.get(customerManagement.getGroupId()).getOrganizationalCode();
                    }
                    String pk_subjcode = "";
                    // 定金
                    if (payLog.getContentType() == 1) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ001;
                    }
                    // 首付
                    if (payLog.getContentType() == 2) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ002;
                    }
                    // 银行按揭
                    if (payLog.getContentType() == 3) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ006;
                    }
                    // 专项维修资金
                    if (payLog.getContentType() == 4) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ015;
                    }
                    // 楼款
                    if (payLog.getContentType() == 5) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ003;
                    }
                    // 补差款
                    if (payLog.getContentType() == 6) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ004;
                    }
                    // 意向金
                    if (payLog.getContentType() == 7) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ026;
                    }

                    String pk_balatype = "";
                    // 现金
                    if (payLog.getPaymentMethod() == 1) {
                        pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_0;
                    }
                    // 支付宝
                    if (payLog.getPaymentMethod() == 2) {
                        pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_4;
                    }
                    // 微信
                    if (payLog.getPaymentMethod() == 3) {
                        pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_4;
                    }
                    // 银行卡
                    if (payLog.getPaymentMethod() == 3) {
                        pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_4;
                    }


                    JSONObject json = new JSONObject();
                    JSONObject ar_gatherbill = new JSONObject();
                    String srcsystemid = payLog.getId();
                    String billdate = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date());
                    ar_gatherbill.put("srcsystemid", srcsystemid); //外系统标识,必选
                    ar_gatherbill.put("pk_tradetype", InvoiceConstant.INVOICE_BUSINESS_F2_Cxx_06); //交易类型,必选
                    ar_gatherbill.put("pk_org", pk_org); //组织编码,必选
                    ar_gatherbill.put("billdate", billdate); //单据日期
                    ar_gatherbill.put("def2", srcsystemid); //外系统单据号，必填，
                    json.put("ar_gatherbill", ar_gatherbill);

                    JSONArray ar_gatheritem = new JSONArray();
                    JSONObject ar_gatheritemJson = new JSONObject();
                    ar_gatheritemJson.put("srcsystemid", srcsystemid); //外系统子表标识
                    ar_gatheritemJson.put("objtype", "0"); //0=客户，1=供应商--判断customer和supplier是否必填，必选
                    ar_gatheritemJson.put("customer", customerManagement.getBuyerIdentityCard()); //客商的纳税人识别号
                    ar_gatheritemJson.put("money_cr", payLog.getPayMoney()); //金额，必选
                    ar_gatheritemJson.put("taxrate", "0"); //税率,必选，数字——3/10/6````
                    ar_gatheritemJson.put("pk_balatype", pk_balatype); //结算方式,必选，看云文档结算方式
                    ar_gatheritemJson.put("pk_recpaytype", "001"); //付款业务类型传默认值“001”
                    ar_gatheritemJson.put("prepay", "1"); //收款性质应收款时传"0",收款性质预收款时传"1"
                    ar_gatheritemJson.put("recaccount", payLog.getBankNumber()); //收款银行账号,必填
                    ar_gatheritemJson.put("scomment", customerManagement.getBuyerName()); //客户名称，必选
                    ar_gatheritemJson.put("pk_subjcode", pk_subjcode); //收款类型
                    ar_gatheritemJson.put("checktype", InvoiceConstant.INVOICE_RECEIPT_TYPE_3); //票据类型，必选，看云文档票据类型
                    ar_gatheritemJson.put("checkno", payLog.getSerialNumber()); //票据号，必选
                    ar_gatheritemJson.put("def9", InvoiceConstant.INVOICE_BUSINESS_FORMAT_001); //业态，必选，看云文档业态
                    ar_gatheritemJson.put("def11", payLog.getHouseId()); //转出前房号编码，收除了诚意金之外的首期等需要选
                    ar_gatheritemJson.put("def10", def10); //项目档案编码，看云文档项目档案对照
                    // TODO: 2024/11/22 银行流水号 待定 暂时不传
//        ar_gatheritemJson.put("top_billid", pkInformerId); //到账通知生单时,（银行流水中返回的id，见5.2.10到账通知认领查询pk_informer字段）
                    ar_gatheritem.add(ar_gatheritemJson);
                    json.put("ar_gatheritem", ar_gatheritem);

                    log.info("收款凭证推送详情:{}", json.toJSONString());

                    String result = InvoiceUtil.gatheringbillAdd(json);
                    JSONObject jsonObject = JSONObject.parseObject(result);
                    if (jsonObject.getString("success").equals("true")) {
                        payLog.setNcSubmitStatus(1);
                    } else {
                        payLog.setNcSubmitStatus(2);
                    }
                    payLog.setNcJson(result);
                    payLog.setNcSubmitDate(new Date());
                    payLog.setNcSubmitName(ShiroUtils.getUser().getTruename());
                    payLogMapper.updateByPrimaryKeySelective(payLog);

                }

            } catch (Exception e) {
                log.error("收款凭证推送报错", e);
            }
        }

        return 1;
    }

    /**
     * 退款记录推送
     *
     * @param collectionDateFrom
     * @param collectionDateTo
     * @return
     */
    @Synchronized
    public int refundRecordSubmit(String ids, String collectionDateFrom, String collectionDateTo) {

        // 开票列表
        Map<String, String> map = new HashMap<>();
        InvoiceManageExample invoiceManageExample = new InvoiceManageExample();
        invoiceManageExample.createCriteria().andTypeEqualTo("1").andInvoiceStatusEqualTo("2");
        List<InvoiceManage> invoiceManages = invoiceManageMapper.selectByExample(invoiceManageExample);
        if (CollectionUtils.isNotEmpty(invoiceManages)) {
            map = invoiceManages.stream().collect(Collectors.toMap(InvoiceManage::getCustomerManagementId, InvoiceManage::getId));
        }

        // 客户列表
        List<CustomerManagement> customerManagements = customerManagementMapper.selectByExample(new CustomerManagementExample());
        Map<String, CustomerManagement> customerMap = customerManagements.stream()
                .collect(Collectors.toMap(CustomerManagement::getId, item -> item));
        // 园区列表
        List<ParkInfo> parkInfos = parkInfoMapper.selectByExample(new ParkInfoExample());
        Map<String, ParkInfo> parkInfoMap = parkInfos.stream().collect(Collectors.toMap(ParkInfo::getId, item -> item));

        // 退款列表
        NccRecordVo model = new NccRecordVo();
        model.setIdList(Arrays.asList(ids.split(",")));
        List<RefundManageDto> refundManages = modelMapper.refundListByIds(model);
        for (RefundManageDto refundManage : refundManages) {
            try {
                if (customerMap.containsKey(refundManage.getCustomerManagementId())) {
                    CustomerManagement customerManagement = customerMap.get(refundManage.getCustomerManagementId());

                    String billdate = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date());
                    // 项目档案编码
                    String def10 = "";
                    if (parkInfoMap.containsKey(customerManagement.getGroupId())) {
                        def10 = parkInfoMap.get(customerManagement.getGroupId()).getArchiveCode();
                    }
                    // 组织机构编码
                    String pk_org = "";
                    if (parkInfoMap.containsKey(customerManagement.getGroupId())) {
                        pk_org = parkInfoMap.get(customerManagement.getGroupId()).getOrganizationalCode();
                    }
                    String pk_subjcode = "";
                    // 定金
                    if (refundManage.getContentType() == 1) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ001;
                    }
                    // 首付
                    if (refundManage.getContentType() == 2) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ002;
                    }
                    // 银行按揭
                    if (refundManage.getContentType() == 3) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ006;
                    }
                    // 专项维修资金
                    if (refundManage.getContentType() == 4) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ015;
                    }
                    // 楼款
                    if (refundManage.getContentType() == 5) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ003;
                    }
                    // 补差款
                    if (refundManage.getContentType() == 6) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ004;
                    }
                    // 意向金
                    if (refundManage.getContentType() == 7) {
                        pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ026;
                    }

                    String checktype = InvoiceConstant.INVOICE_RECEIPT_TYPE_3;
                    // 如对应项目的客户，未开预收款发票的退款，则为收据；开了预收款发票则为发票
                    if (map.containsKey(refundManage.getCustomerManagementId())) {
                        checktype = InvoiceConstant.INVOICE_RECEIPT_TYPE_2;
                    }


                    JSONObject json = new JSONObject();
                    JSONObject ap_paybill = new JSONObject();
                    String srcsystemid = IdUtil.simpleUUID();
                    ap_paybill.put("srcsystemid", srcsystemid); //外系统标识,必选
                    ap_paybill.put("pk_tradetype", InvoiceConstant.INVOICE_BUSINESS_F3_Cxx_06); //交易类型,必选
                    ap_paybill.put("pk_org", pk_org); //组织编码,必选
                    ap_paybill.put("billdate", billdate); //单据日期
                    ap_paybill.put("def2", srcsystemid); //外系统单据号，必填，
                    json.put("ap_paybill", ap_paybill);

                    JSONArray ap_payitem = new JSONArray();
                    JSONObject ap_payitemJson = new JSONObject();
                    ap_payitemJson.put("srcsystemid", srcsystemid); //外系统子表标识
                    ap_payitemJson.put("scomment", customerManagement.getBuyerName()); //客户名称，必选
                    ap_payitemJson.put("pk_subjcode", pk_subjcode); //收支项目编码，必选
                    ap_payitemJson.put("objtype", "0"); //0=客户，1=供应商--判断customer和supplier是否必填，必选
                    ap_payitemJson.put("customer", customerManagement.getBuyerIdentityCard()); //客商的纳税人识别号
                    ap_payitemJson.put("money_de", refundManage.getActualRefundAmount()); //金额，必选
                    ap_payitemJson.put("taxrate", "0"); //税率，必选，无税传0
                    ap_payitemJson.put("pk_balatype", "4"); //结算方式，必选,默认是4
                    ap_payitemJson.put("pk_recpaytype", "001"); //付款业务类型传默认值“001”
                    ap_payitemJson.put("prepay", "0"); //默认值0
                    ap_payitemJson.put("payaccount", refundManage.getBankNumber()); //付款银行账号，必选
                    ap_payitemJson.put("checktype", checktype); //票据类型，必选，看云文档票据类型
                    // TODO: 2024/11/22 退款的记录 是汇合的多笔的 票据号待定
                    ap_payitemJson.put("checkno", IdUtil.simpleUUID()); //票据号，必选
                    ap_payitemJson.put("def11", refundManage.getHouseId()); //转出前房号编码，必选
                    ap_payitemJson.put("def9", "001"); //业态，必选，看云文档业态
                    ap_payitemJson.put("def10", def10); //项目档案编码，看云文档项目档案对照
                    ap_payitem.add(ap_payitemJson);
                    json.put("ap_payitem", ap_payitem);

                    log.info("退款凭证推送详情:{}", json.toJSONString());

                    RefundDetail detail = new RefundDetail();
                    detail.setId(refundManage.getRefundDetailId());

                    String result = InvoiceUtil.paybillAdd(json);
                    JSONObject jsonObject = JSONObject.parseObject(result);

                    if (jsonObject.getString("success").equals("true")) {
                        detail.setNcSubmitStatus(1);
                    } else {
                        detail.setNcSubmitStatus(2);
                    }
                    detail.setNcJson(result);
                    detail.setNcSubmitDate(new Date());
                    detail.setNcSubmitName(ShiroUtils.getUser().getTruename());
                    refundDetailMapper.updateByPrimaryKeySelective(detail);
                }

            } catch (Exception e) {
                log.error("退款凭证推送报错", e);
            }

        }
        return 1;

    }

    /**
     * 转换记录 ncc提交
     *
     * @param collectionDateFrom
     * @param collectionDateTo
     * @return
     */
    @Synchronized
    public int convertRecordSubmit(String ids, String collectionDateFrom, String collectionDateTo) {

        // 客户列表
        List<CustomerManagement> customerManagements = customerManagementMapper.selectByExample(new CustomerManagementExample());
        Map<String, CustomerManagement> customerMap = customerManagements.stream()
                .collect(Collectors.toMap(CustomerManagement::getId, item -> item));
        // 园区列表
        List<ParkInfo> parkInfos = parkInfoMapper.selectByExample(new ParkInfoExample());
        Map<String, ParkInfo> parkInfoMap = parkInfos.stream().collect(Collectors.toMap(ParkInfo::getId, item -> item));

        // 转换列表
        NccRecordVo model = new NccRecordVo();
        model.setIdList(Arrays.asList(ids.split(",")));

        // 转换记录
        List<NccRecordVo> list = modelMapper.convertRecord(model);
        for (NccRecordVo record : list) {
            // 剔除已提交的记录
            if (record.getNcSubmitStatus() != 1) {
                convertRecordSubmitInfo(record,customerMap,parkInfoMap);
            }
        }
        return 1;

    }

    public void convertRecordSubmitInfo(NccRecordVo record, Map<String, CustomerManagement> customerMap, Map<String, ParkInfo> parkInfoMap) {

        String customerManagementId = "";
        Integer paymentMethod = null;
        String money_cr = "";
        String bankNumber = "";
        String houseId = "";
        String serialNumber = "";
        // 意向金转定金
        if (record.getConvertType().equals("1")) {
            IntentionalDeposit intentionalDeposit = intentionalDepositMapper.selectByPrimaryKey(record.getBusinessId());
            customerManagementId = intentionalDeposit.getCustomerManagementId();
            paymentMethod = intentionalDeposit.getPaymentMethod();
            money_cr = intentionalDeposit.getReceivedAmount().toString();
            bankNumber = intentionalDeposit.getBankNumber();
            houseId = intentionalDeposit.getHouseId();
            serialNumber = intentionalDeposit.getSerialNumber();
        }
        // 定金转首款 定金转房款
        if (record.getConvertType().equals("2") || record.getConvertType().equals("3")) {
            PayLog payLog = payLogMapper.selectByPrimaryKey(record.getBusinessId());
            customerManagementId = payLog.getCustomerManagementId();
            paymentMethod = payLog.getPaymentMethod();
            money_cr = payLog.getPayMoney().toString();
            bankNumber = payLog.getBankNumber();
            houseId = payLog.getHouseId();
            serialNumber = payLog.getSerialNumber();
        }

        try {
            String billdate = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date());
            if (customerMap.containsKey(customerManagementId)) {
                CustomerManagement customerManagement = customerMap.get(customerManagementId);

                // 项目档案编码
                String def10 = "";
                if (parkInfoMap.containsKey(customerManagement.getGroupId())) {
                    def10 = parkInfoMap.get(customerManagement.getGroupId()).getArchiveCode();
                }
                // 组织机构编码
                String pk_org = "";
                if (parkInfoMap.containsKey(customerManagement.getGroupId())) {
                    pk_org = parkInfoMap.get(customerManagement.getGroupId()).getOrganizationalCode();
                }

                // 目前只有意向金转定金
                String pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ001;

                String pk_balatype = "";
                // 现金
                if (paymentMethod == 1) {
                    pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_0;
                }
                // 支付宝
                if (paymentMethod == 2) {
                    pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_4;
                }
                // 微信
                if (paymentMethod == 3) {
                    pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_4;
                }
                // 银行卡
                if (paymentMethod == 3) {
                    pk_balatype = InvoiceConstant.INVOICE_PAYMENT_METHOD_4;
                }


                JSONObject json = new JSONObject();
                JSONObject ar_gatherbill = new JSONObject();
                String srcsystemid = record.getId();
                ar_gatherbill.put("srcsystemid", srcsystemid); //外系统标识,必选
                ar_gatherbill.put("pk_tradetype", InvoiceConstant.INVOICE_BUSINESS_F2_Cxx_07); //交易类型,必选
                ar_gatherbill.put("pk_org", pk_org); //组织编码,必选
                ar_gatherbill.put("billdate", billdate); //单据日期
                ar_gatherbill.put("def2", srcsystemid); //外系统单据号，必填，
                json.put("ar_gatherbill", ar_gatherbill);

                JSONArray ar_gatheritem = new JSONArray();
                JSONObject ar_gatheritemJson = new JSONObject();
                ar_gatheritemJson.put("srcsystemid", srcsystemid); //外系统子表标识
                ar_gatheritemJson.put("objtype", "0"); //0=客户，1=供应商--判断customer和supplier是否必填，必选
                ar_gatheritemJson.put("customer", customerManagement.getBuyerIdentityCard()); //客商的纳税人识别号
                ar_gatheritemJson.put("money_cr", money_cr); //金额，必选
                ar_gatheritemJson.put("taxrate", "0"); //税率,必选，数字——3/10/6````
                ar_gatheritemJson.put("pk_balatype", pk_balatype); //结算方式,必选，看云文档结算方式
                ar_gatheritemJson.put("pk_recpaytype", "001"); //付款业务类型传默认值“001”
                ar_gatheritemJson.put("prepay", "0"); //收款性质应收款时传"0",收款性质预收款时传"1"
                ar_gatheritemJson.put("recaccount", bankNumber); //收款银行账号,必填
//        ar_gatheritemJson.put("invoiceno","fphm"); //发票号码，非必选
                ar_gatheritemJson.put("scomment", customerManagement.getBuyerName()); //客户名称，必选
                ar_gatheritemJson.put("pk_subjcode", pk_subjcode); //收支项目编码，必选，看云文档收支项目对照
                ar_gatheritemJson.put("def11", houseId); //转出前房号编码，必选
                ar_gatheritemJson.put("def9", InvoiceConstant.INVOICE_BUSINESS_FORMAT_001); //业态，必选，看云文档业态
//        ar_gatheritemJson.put("def12", ""); //转出后房号编码，换房业务时要选
                ar_gatheritemJson.put("def14", InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ026); //原收支项目编码，定金转违约金项目时 必选
//        ar_gatheritemJson.put("def13", ""); //换票后票据号，换房业务必选
                ar_gatheritemJson.put("checktype", InvoiceConstant.INVOICE_RECEIPT_TYPE_3); //票据类型，必选，看云文档票据类型
                ar_gatheritemJson.put("checkno", serialNumber); //票据号，必选
                ar_gatheritemJson.put("def10", def10); //项目档案编码，看云文档项目档案对照
                ar_gatheritem.add(ar_gatheritemJson);
                json.put("ar_gatheritem", ar_gatheritem);

                log.info("转换记录推送详情:{}", json.toJSONString());
                String result = InvoiceUtil.gatheringbillTurn(json);
                JSONObject jsonObject = JSONObject.parseObject(result);
                ConvertRecord convertRecord = convertRecordMapper.selectByPrimaryKey(record.getId());
                if (jsonObject.getString("success").equals("true")) {
                    convertRecord.setNcSubmitStatus(1);
                } else {
                    convertRecord.setNcSubmitStatus(2);
                }
                convertRecord.setNcJson(result);
                convertRecord.setNcSubmitDate(new Date());
                convertRecord.setNcSubmitName(ShiroUtils.getUser().getTruename());
                convertRecordMapper.updateByPrimaryKeySelective(convertRecord);
            }

        } catch (Exception e) {
            log.error("转换记录凭证推送报错", e);
        }


    }


}
