package com.idea.workPanel.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.workPanel.enume.HandleTypeEnum;
import com.idea.workPanel.mapper.ParkWaitHandleMapper;
import com.idea.workPanel.mapper.ParkWaitHandleOnMapper;
import com.idea.workPanel.model.ParkWaitHandle;
import com.idea.workPanel.model.ParkWaitHandleExample;
import com.idea.workPanel.model.ParkWaitHandleOn;
import com.idea.workPanel.model.ParkWaitHandleOnExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.FramePermission;
import com.rockstar.frame.model.FramePermissionExample;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FramePermissionService;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ParkWaitHandleService implements BaseService<ParkWaitHandle, ParkWaitHandleExample> {
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private ParkWaitHandleMapper modelMapper;
    @Autowired
    private ParkWaitHandleOnMapper modelOnMapper;
    @Autowired
    private FramePermissionService framePermissionService;

    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        ParkWaitHandleExample example = new ParkWaitHandleExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public ParkWaitHandle selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(ParkWaitHandle record) {
        if (StringUtils.isEmpty(record.getId())) {
            record.setId(SnowflakeIdWorker.getUUID());
            record.setCreatedAt(new Date());
            return modelMapper.insert(record);
        }
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(ParkWaitHandle record, ParkWaitHandleExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(ParkWaitHandle record, ParkWaitHandleExample example) {
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<ParkWaitHandle> selectByExample(ParkWaitHandleExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(ParkWaitHandleExample example) {
        return modelMapper.countByExample(example);
    }

    @Override
    public int deleteByExample(ParkWaitHandleExample example) {
        return modelMapper.deleteByExample(example);
    }


    public int insertWithoutId(ParkWaitHandle record) {
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(ParkWaitHandle record) {
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insertSelective(record);
    }

    public int insert(ParkWaitHandle record) {
//        SnowflakeIdWorker idWorker = new SnowflakeIdWorker(0L, 0L);
//        long id = idWorker.nextId();
        record.setId(IdUtil.objectId());
        return modelMapper.insert(record);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkWaitHandle> list(Tablepar tablepar, ParkWaitHandle model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkWaitHandle> list = modelMapper.selectByExample(getCondition(model, dt));
        PageInfo<ParkWaitHandle> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<ParkWaitHandle> listAll(ParkWaitHandle model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    public ParkWaitHandle getParkWaitHandle(ParkWaitHandle model) {
        ParkWaitHandle ParkWaitHandle = new ParkWaitHandle();
        List<ParkWaitHandle> ParkWaitHandles = modelMapper.selectByExample(getCondition(model, null));
        if (ParkWaitHandles != null && ParkWaitHandles.size() > 0) {
            ParkWaitHandle = ParkWaitHandles.get(0);
        }
        return ParkWaitHandle;
    }

    private ParkWaitHandleExample getCondition(ParkWaitHandle model, DateTrans dt) {
        ParkWaitHandleExample ex = new ParkWaitHandleExample();
        if (null == model) {
            return ex;
        }
        ex.setOrderByClause("created_at desc");
        ParkWaitHandleExample.Criteria criteria = ex.createCriteria();

        return ex;
    }

    // 获取待办列表
    public PageInfo<ParkWaitHandle> getMyHandleList(Tablepar tablepar, Integer type) {
        ParkWaitHandleOnExample onExample = new ParkWaitHandleOnExample();
        onExample.createCriteria().andHandleUserIdEqualTo(ShiroUtils.getUserId());
        List<ParkWaitHandleOn> onList = modelOnMapper.selectByExample(onExample);
        List<String> onStr = onList.stream().map(ParkWaitHandleOn::getHandleId).collect(Collectors.toList());
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        ParkWaitHandleExample example = new ParkWaitHandleExample();
        example.setOrderByClause("created_at desc");
        ParkWaitHandleExample.Criteria criteria = example.createCriteria();
        criteria.andReceiveUserIdEqualTo(ShiroUtils.getUserId());
        if(null != onStr && onStr.size() > 0)
            criteria.andIdNotIn(onStr);
        if(null != type)
            criteria.andHandleTypeEqualTo(type);
        List<ParkWaitHandle> list = modelMapper.selectByExample(example);
        PageInfo<ParkWaitHandle> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    // 获取待办列表
    public PageInfo<ParkWaitHandle> getMyHandleListToMoblie(Tablepar tablepar, String types, String userId) {
        ParkWaitHandleOnExample onExample = new ParkWaitHandleOnExample();
        onExample.createCriteria().andHandleUserIdEqualTo(userId);
        List<ParkWaitHandleOn> onList = modelOnMapper.selectByExample(onExample);
        List<String> onStr = onList.stream().map(ParkWaitHandleOn::getHandleId).collect(Collectors.toList());
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        ParkWaitHandleExample example = new ParkWaitHandleExample();
        example.setOrderByClause("created_at desc");
        ParkWaitHandleExample.Criteria criteria = example.createCriteria();
//        criteria.andReceiveUserIdEqualTo(userId);
        if(StringUtils.isNotEmpty(userId)){
            FrameUser user = frameUserService.selectByPrimaryKey(userId);
            if(null != user){
                criteria.andReceiveUserIdIn(CollUtil.newArrayList("0", userId, user.getDataRole()));
            }
        }
        if(null != onStr && onStr.size() > 0)
            criteria.andIdNotIn(onStr);
        if(StringUtils.isNotEmpty(types)){
            List<Integer> typeList = Arrays.stream(types.split(",")).map(item -> Integer.parseInt(item)).collect(Collectors.toList());
            criteria.andHandleTypeIn(typeList);
        }
        List<ParkWaitHandle> list = modelMapper.selectByExample(example);
        PageInfo<ParkWaitHandle> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    // 获取待办数量
    public Integer getMyHandleSize(String userId, Integer type) {
        ParkWaitHandleOnExample onExample = new ParkWaitHandleOnExample();
        onExample.createCriteria().andHandleUserIdEqualTo(ShiroUtils.getUserId());
        List<ParkWaitHandleOn> onList = modelOnMapper.selectByExample(onExample);
        List<String> onStr = onList.stream().map(ParkWaitHandleOn::getHandleId).collect(Collectors.toList());
        ParkWaitHandleExample example = new ParkWaitHandleExample();
        ParkWaitHandleExample.Criteria criteria = example.createCriteria();
        criteria.andReceiveUserIdEqualTo(userId);
        if(null != onStr && onStr.size() > 0)
            criteria.andIdNotIn(onStr);
        if(null != type)
            criteria.andHandleTypeEqualTo(type);
        Long size = modelMapper.countByExample(example);
        return null==size?0:size.intValue();
    }

    // 处理
    @Transactional
    public void userHandle(HandleTypeEnum typeEnum, String param, String userId) {
        if(!StringUtils.isNotEmpty(param))
            return;
        ParkWaitHandleExample example = new ParkWaitHandleExample();
        example.createCriteria().andHandleTypeEqualTo(typeEnum.getValue()).andRouteParamEqualTo(param);
        List<ParkWaitHandle> handleList = modelMapper.selectByExample(example);
        if(null == handleList || handleList.size() <= 0)
            return;
        SnowflakeIdWorker idWorker = new SnowflakeIdWorker(0L, 0L);
        for (ParkWaitHandle parkWaitHandle : handleList) {
            ParkWaitHandleOn on = new ParkWaitHandleOn();
            on.setId(idWorker.genUUID());
            on.setHandleId(parkWaitHandle.getId());
//            on.setHandleUserId(userId);
            on.setHandleUserId(parkWaitHandle.getReceiveUserId());
            on.setHandleTime(new Date());
            modelOnMapper.insert(on);
        }
    }

    /**
     * 添加待办
     * @param title 显示标题
     * @param typeEnum 处理类型枚举
     * @param srcUserType 来源用户类型 0管理,1企业
     * @param srcUserId 来源用户id
     * @param srcUserName 来源用户名称
     * @param receiveUserId 接收用户id or 0=全部
     * @param routeParam 跳转携带参数，例： id=***&type=**
     */
    public void addHandle(String title, HandleTypeEnum typeEnum, Integer srcUserType, String srcUserId, String srcUserName,
                          String receiveUserId, String routeParam) {
        ParkWaitHandle handle = new ParkWaitHandle();
//        handle.setCreateBy(ShiroUtils.getUserId());
        handle.setCreatedAt(new Date());
        handle.setHandleType(typeEnum.getValue());
        handle.setRouteUrl(typeEnum.getUrl());
        handle.setTitle(title);
        handle.setSrcUserId(srcUserId);
        handle.setSrcUserType(srcUserType);
        handle.setSrcUserId(srcUserId);
        handle.setSrcUserName(srcUserName);
        handle.setReceiveUserId(receiveUserId);
        handle.setRouteParam(routeParam);
        insert(handle);


//        //发送短信
//        FrameUser user = frameUserService.selectByPrimaryKey(receiveUserId);
//        if(StringUtils.isNotEmpty(user.getPhone())){
//            try {
//                SmsSendUtil.sendText(title,user.getPhone());
//            } catch (Exception e) {
//                log.error("短信发送异常",e);
//                e.printStackTrace();
//            }
//        }

    }

    public String getUrl(String parentName, String childName){
        String waitUrl = "";
        // 查询待办页面的 菜单id
        FramePermissionExample framePermissionExample = new FramePermissionExample();
        framePermissionExample.createCriteria().andNameEqualTo(parentName);
        List<FramePermission> framePermissions = framePermissionService.selectByExample(framePermissionExample);
        if (CollectionUtils.isNotEmpty(framePermissions)) {
            String menuIndex = framePermissions.get(0).getId();
            FramePermissionExample permissionExample = new FramePermissionExample();
            permissionExample.createCriteria().andNameEqualTo(childName);
            List<FramePermission> permissions = framePermissionService.selectByExample(framePermissionExample);
            if (CollectionUtils.isNotEmpty(permissions)) {
                String subMenu = permissions.get(0).getId();
                waitUrl = "/backstage?menuIndex=" + menuIndex + "&subMenu=" + subMenu;
            }

        }
        return waitUrl;
    }



}