package com.idea.pro.wx.controller.buildManage;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.dto.ParkFloorDiscDto;
import com.idea.buildManage.excel.ParkFloorDiscExcel;
import com.idea.buildManage.excel.ParkFloorDiscListener;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.response.ParkFloorDiscRespson;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.idea.util.ExcelUtils;
import com.rockstar.common.base.BaseController;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.system.service.SysDictService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * Created by pengyq on 2020.
 */
@Controller
@RequestMapping(value = "/wx/ParkFloorDiscController")
@Api(value="楼盘分期管理")
public class WxParkFloorDiscController extends BaseController {

    //主表
    @Autowired
    private ParkFloorDiscService modelService;
    @Autowired
    private ParkInfoService parkInfoService;
    @Autowired
    private SysDictService sysDictService;

    @PostMapping(value = "list",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object list(Tablepar tablepar, ParkFloorDiscDto model, DateTrans dt){
        PageInfo<ParkFloorDisc> page= modelService.list(tablepar,model,dt);
        TableSplitResult<ParkFloorDisc> result = new TableSplitResult<ParkFloorDisc>(page.getPageNum(), page.getTotal(), page.getList());
        return  result;
    }

    @ApiOperation(value = "楼盘分期列表")
    @PostMapping(value = "listAll",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object listAll(ParkFloorDiscDto model, DateTrans dt){
        return  modelService.listAll(model,dt);
    }

    @ApiOperation(value = "新增楼盘分期")
    @PostMapping(value = "add",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult add(ParkFloorDiscRespson record){
        record.setCreatedAt(new Date());
        int result = modelService.add(record);
        return result(result);
    }

    @ApiOperation(value = "删除楼盘分期")
    @PostMapping(value = "remove/{id}",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult remove(@PathVariable("id") String id){
        int result = modelService.deleteByPrimaryKey(id);
        return result(result);
    }

    @ApiOperation(value = "编辑楼盘分期")
    @PostMapping(value = "edit",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult editSave(ParkFloorDiscRespson model){
        model.setUpdatedAt(new Date());
        int result = modelService.edit(model);
        return result(result);
    }

    @ApiOperation(value = "根据id查询楼盘分期")
    @PostMapping(value = "getById",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public ParkFloorDiscRespson getById(String id) {
        return modelService.getById(id);
    }

    @ApiOperation(value = "楼盘分期分页")
    @PostMapping(value = "listByModel",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object listByModel(Tablepar tablepar, ParkFloorDiscRespson model, DateTrans dt){
        PageInfo<ParkFloorDiscRespson> page= modelService.listByModel(tablepar,model,dt);
        TableSplitResult<ParkFloorDiscRespson> result = new TableSplitResult<ParkFloorDiscRespson>(page.getPageNum(), page.getTotal(), page.getList());
        return  result;
    }

    /**
     * 分期导入
     * @param file
     * @return
     */
    @PostMapping(value = "handleImport", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult handleImport(@RequestParam("file") MultipartFile file) {
        JSONObject jsonObject = new JSONObject();
        ParkFloorDiscListener listener = new ParkFloorDiscListener(modelService,parkInfoService,sysDictService,jsonObject);
        try {
            ExcelUtils excelUtils = new ExcelUtils();
            File excel = excelUtils.multipartFileToFile(file);
            if (null == excel) {
                return AjaxResult.error("导入失败");
            }
            EasyExcel.read(excel, ParkFloorDiscExcel.class, listener).headRowNumber(1).sheet().doRead();
            excel.delete();
//            List<ParkFloorDiscExcel> errorList = (List<ParkFloorDiscExcel>) jsonObject.get("errorList");
//            modelService.errorListExport(errorList,response);
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error(e.getMessage());
        }
        return AjaxResult.success(jsonObject);
    }

    /**
     * 导出
     * @param list
     * @param response
     * @throws IOException
     */
    @PostMapping(value = "errorListExport")
    @ResponseBody
    public void errorListExport(@RequestBody List<ParkFloorDiscExcel> list, HttpServletResponse response) throws IOException {
        modelService.errorListExport(list,response);
    }


    @ApiOperation(value = "分期删除")
    @PostMapping(value = "delete",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult delete(String id) {
        int result = modelService.delete(id);
        return result(result);
    }


}
